static char *sccsid = "@(#) cmd/mqlink/amqslnk0.c, mqlink, nt200, nt200-csd08          1.19.1.3  97/02/19 09:40:14";
/********************************************************************/
/*                                                                  */
/* Program name: AMQSLNK0                                           */
/*                                                                  */
/* Description: Sample MQSeries C program which accepts the         */
/*              Lotus Notes Client data from the mqlink program     */
/*                                                                  */
/* Statement:     Licensed Materials - Property of IBM              */
/*                                                                  */
/*                33H2205, 5622-908                                 */
/*                33H2267, 5765-623                                 */
/*                29H0990, 5697-176                                 */
/*                (C) Copyright IBM Corp. 1994, 1995                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Function:                                                        */
/*                                                                  */
/* This program provides the backend MQSeries support for an        */
/* "MQSeries Sample" Lotus Notes database which demonstrates the    */
/* function of the MQSeries Lotus Notes server exit.                */
/*                                                                  */
/* amqslnk0 is a sample MQSeries C program which receives data      */
/* from the input queue ( default - SYSTEM.SAMPLE.NOTES.INQUEUE )   */
/* and replies to the reply queue specified in the MQENTRY document */
/* of the link database.                                            */
/*                                                                  */
/* The "MQSeries Sample" database contains a forms from which a     */
/* document can be composed.                                        */
/*                                                                  */
/* 1) A message document should be used as follows :-               */
/* The data typed in the 'data to be sent' field will be sent to    */
/* the Mail_In database when the Send data button is pressed.       */
/* It will then be read from the Mail_In database by MQLINK and     */
/* placed on the MQSeries target queue specified in the link        */
/* database document MQENTRY.                                       */
/* The message will then be retrieved by amqslnk0.                  */
/* the text tokenised, reversed and placed on the reply queue.      */
/* MQLINK will then remove this reply message and update the        */
/* originating document.                                            */
/* If the Message is "Quit" the program terminates.                 */
/*                                                                  */
/* A document composed for the Lotus Notes database                 */
/* 'MQSeries Sample' (amqsampl.nsf) contains a                      */
/* 'Send data' button which issues a @MailSend() to a Mail-In       */
/* database. The MQseries program  "mqlink"    examines the data    */
/* and places it on 'SYSTEM.SAMPLE.NOTES.INQUEUE'.                  */
/* This program   must have previously been started and sits        */
/* in a Get wait for a message to arrive.                           */
/*                                                                  */
/* The Sample database Views are :-                                 */
/*    Sample View - default - message and Tic-Tac-Toe               */
/*                                                                  */
/* The Sample database documents are :-                             */
/*    MQSeries Sample    - simple message                           */
/*                                                                  */
/*                                                                  */
/*  Procedures :-                                                   */
/*   GetParm    - get parameters                                    */
/*   InquireQmgr - Inquire Queue Manager name                       */
/*   Msg_call   - Analyse Msg message                               */
/*   main       - Main procedure                                    */
/*                                                                  */
/*                                                                  */
/*  Returns - 0 - OK                                                */
/*            1 - MQGET failed                                      */
/*            2 - Invalid function value                            */
/*                                                                  */
/*  Program logic:                                                  */
/*     MQCONNect to message queue manager                           */
/*     MQOPEN message queue (Input) for shared input                */
/*                                                                  */
/*     .  MQGET next message from queue Input queue                 */
/*     .  if message is a "Msg" request,                            */
/*     .  .  MQPUT1 to the REPLY queue                              */
/*     MQCLOSE Input queue                                          */
/*     MQDISConnect from queue manager                              */
/*                                                                  */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/*   amqslnk0 has 2 parameters - an input queue name                */
/*                             - a queue manager name               */
/*   The defaults are :-                                            */
/*       'SYSTEM.SAMPLE.NOTES.INQUEUE'                              */
/*       default queue manager                                      */
/*                                                                  */
/*   The reply queue name is specified in the link database         */
/*                                                                  */
/*   The syntax is :-                                               */
/*             amqslnk0 [-q InputQName] [QMgrName]                  */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/*  e.g. crtmqm -q qmgrname { create queue manager              }   */
/*       strmqm             { start queue manager               }   */
/*      (runmqsc qmgrname <amqscoma.tst { define default queues })  */
/*       runmqsc qmgrname <amqslnk0.tst { define Samples queues }   */
/*       amqslnk0           { start this program                }   */
/*      (endmqm -i qmgrname { end queue manager                 })  */
/*                                                                  */
/********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

/*  includes for MQI                                                 */
#include <cmqc.h>
	
#define token_sep " \n\r"          /* Separators for words           */

struct Sample_Msg {                /* structure for message buffer   */
	      char function[3];    /* "Msg"                          */
	      char Msg[102];       /* Message                        */
	      char reserved[23];   /* make same length as reply      */
	          };

struct Sample_Rep {                /* structure for reply buffer     */
	      char Msg[102];       /* Message                        */
	      char Time[26];       /* process time                   */
	          };
	

int Msg_call(void);                /* Prototype for Borland C++      */
void InquireQmgr(MQCHAR48 QmgrN);

  /*   Declare MQI structures needed                                 */
  MQOD    odT = {MQOD_DEFAULT};    /* Object Descriptor for temp Q   */
  MQOD    odI = {MQOD_DEFAULT};    /* Object Descriptor for input Q  */
  MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor             */
  MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options            */
  MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options            */
	
  MQHCONN  Hcon = 0;               /* connection handle              */
  MQHOBJ   Hinp;                   /* object handle, input queue     */
  MQHOBJ   Htmp;                   /* object handle, temporary queue */
  MQLONG   O_options;              /* MQOPEN options                 */
  MQLONG   C_options;              /* MQCLOSE options                */
  MQLONG   CompCode = 0;           /* completion code                */
  MQLONG   Reason   = 0;           /* reason code                    */
  MQLONG   CReason  = 0;           /* reason code (MQCONN)           */
  char     reply[100];             /* message reply text             */
  struct Sample_Msg buffer;        /* message buffer                 */
  struct Sample_Rep Repbuffer;     /* message reply buffer           */
  MQLONG   buflen;                 /* buffer length                  */
  MQLONG   messlen;                /* message length received        */
  MQCHAR48 QMName = " ";           /* queue manager name - default   */
  MQCHAR48 iQueue = "SYSTEM.SAMPLE.NOTES.INQUEUE"; /* Input queue    */


/*********************************************************************/
/*                                                                   */
/* Function Name:  GetParm                                           */
/*                                                                   */
/* Description:    Analyse the program parameters                    */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/* Analyse the program parameters                                    */
/*                                                                   */
/* Input Parameters:                                                 */
/*       argc - count of parameters + 1                              */
/*       argv - array of string values                               */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:           integer                                        */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Exit Normal:       1 - have analysed parameters                   */
/*                                                                   */
/* Exit Error:        0 - no parameters                              */
/*                                                                   */
/*********************************************************************/

unsigned short GetParm(int argc, char *argv[])
{
#define Nokey      1               /* No. keyword parameters         */

 char *Parmkey[Nokey] = { "-q" };  /* keywords allowed               */
 int i,j;

	
 if (argc >1)                      /* are any parameters given ?     */
 {
  for (i=1; i<argc; i++)
   {
    for (j=0; j<Nokey; j++)        /* look for one of the keywords   */
     {
      if (!strncmp( argv[i], Parmkey[j], 2)) break; /* .. found      */
     };                            /* end of for j                   */

    switch (j)                     /* Jump to correct keyword        */
     {
      case 0:                      /* -m parameter                   */
       {
	if (i+1 < argc)
	{
	  strncpy( iQueue, argv[i+1], sizeof(iQueue) );
	  i++;
	};
	
	break;
       };                          /* end of case 0                  */
	
      default:
       {                           /* must be queue manager name     */
	strncpy( QMName, argv[i], sizeof(iQueue) );
       };                          /* end of default                 */
     };                            /* end of switch                  */
   };                              /* end of for i                   */
 }
 else
  return(0);                       /* No parameters given            */

 return(1);                        /* Have anlysed parameters        */
}                                  /* end of GetParm                 */

/*********************************************************************/
/*                                                                   */
/* Function Name:  InquireQmgr                                       */
/*                                                                   */
/* Description:    Find Queue Manager Name                           */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/* Issue MQINQ to find queue manager name                            */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Input Parameters:                                                 */
/*      Pointer to Queue manager name area                           */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*                    Queue Manager name                             */
/*                                                                   */
/* Exit Normal:       Queue manager name set                         */
/*                                                                   */
/* Exit Error:        none                                           */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Notes:                                                            */
/*   Restrictions:    None                                           */
/*                                                                   */
/*********************************************************************/

void InquireQmgr(MQCHAR48 QmgrN)
{
 MQLONG   Select[1];
 MQLONG   O_options;              /* MQOPEN options                 */
 MQLONG   C_options;              /* MQCLOSE options                */
 MQHOBJ   Hqm;                    /* QM object handle               */
 MQOD     qmod = {MQOD_DEFAULT};  /* QManager Object Descriptor     */
 MQLONG   condCode, reason;

 O_options = MQOO_INQUIRE;
 qmod.ObjectType = MQOT_Q_MGR;
 MQOPEN( Hcon,                    /* connection handle              */
	 &qmod,                   /* object descriptor for QM       */
	 O_options,               /* open options                   */
	 &Hqm,                    /* QM object handle               */
	 &condCode,               /* completion code                */
	 &reason );               /* reason code                    */


 Select[0] = MQCA_Q_MGR_NAME;
 MQINQ( Hcon,                     /* connection handle              */
	Hqm,                      /* QM object handle               */
	1L, Select,               /* selector                       */
	0L, NULL,                 /* no integer attributes          */
	MQ_Q_MGR_NAME_LENGTH, QmgrN,  /* char attributes            */
	&condCode,                /* completion code                */
	&reason );                /* reason code                    */


 C_options = 0;                   /* no close options               */
 MQCLOSE( Hcon,                   /* connection handle              */
	  &Hqm,                   /* QM object handle               */
	  C_options,
	  &condCode,              /* completion code                */
	  &reason );              /* reason code                    */

}                                 /* End of InquireQmgr             */


/*********************************************************************/
/*                                                                   */
/* Function Name:  Msg_call                                          */
/*                                                                   */
/* Description:     Analyse a "Msg" record.                          */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/* Analyse a "Msg" record.                                           */
/*                                                                   */
/* Input Parameters:                                                 */
/*       none                                                        */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:   Reason                                                 */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Exit Normal:       Reason                                         */
/*                                                                   */
/* Exit Error:        Reason                                         */
/*                                                                   */
/*********************************************************************/

int Msg_call(void)
{
 char *token;
 int i,j, tokenlen;
 time_t ltime;

 md.Persistence = MQPER_PERSISTENT;
 memcpy(md.CorrelId, md.MsgId, sizeof(MQBYTE24)); /* copy MQGET value*/
	

 j = sizeof(Repbuffer.Msg);
 memset(Repbuffer.Msg, ' ', j);

 i = strlen(buffer.Msg);          /* length of passed message       */
 j = i;                           /* initialize length              */

	                          /* break up msg. into words and   */
 token = strtok( buffer.Msg, token_sep ); /* in to the reply buffer */
 if (token)                       /* are there any tokens ?         */
  do
   {
    tokenlen = strlen(token);
    j = j - tokenlen;               /* Offset to insert word          */
    if (j < 0) break;
    strcpy( &Repbuffer.Msg[j], token );
    Repbuffer.Msg[j+tokenlen] = ' ';
    j--;                            /* before token/word              */
    if (j < 0) break;
    token = strtok( NULL, token_sep );
   } while (token);


 for (i=j; i>0; i--)              /* clear rest of area             */
  Repbuffer.Msg[i] = ' ';

 time(&ltime);               /* Now add the time to the reply buffer*/
 strcpy( Repbuffer.Time, ctime(&ltime) );
 Repbuffer.Time[24] = ' ';        /* Overwrite NL character         */

 memcpy(odT.ObjectName, md.ReplyToQ, MQ_Q_NAME_LENGTH);
 memcpy(odT.ObjectQMgrName, md.ReplyToQMgr, MQ_Q_MGR_NAME_LENGTH);
	
 messlen = sizeof(struct Sample_Rep); /* Set size of reply message  */

 printf("About to put to reply queue=%.48s,\n Qmgr=%.48s.\n,"\
	 "messlen=%d,\n",
	 odT.ObjectName,odT.ObjectQMgrName,messlen);

 md.Persistence = MQPER_PERSISTENT;
 /* the correlation id is used to tie the msg. to the reply         */
 memcpy(md.CorrelId, md.MsgId, sizeof(md.CorrelId));

 printf("Repbuffer.Msg=%.32s.\n",Repbuffer.Msg);
 printf("Repbuffer.Time=%.26s.\n",Repbuffer.Time);

 /*******************************************************************/
 /*                                                                 */
 /*   Put the message to the reply queue                            */
 /*                                                                 */
 /*******************************************************************/
 MQPUT1(Hcon,            /* connection handle                       */
	&odT,            /* object descriptor                       */
	&md,             /* message descriptor                      */
	&pmo,            /* default options                         */
	messlen,         /* message length                          */
	&Repbuffer,      /* message buffer                          */
	&CompCode,       /* completion code                         */
	&Reason);        /* reason code                             */
	
 if (!strncmp(buffer.Msg, "Quit", 4)) /* Exit if requested          */
  return(-1);

 return(Reason);
}                                 /* end of Msg_call                */
	




/*********************************************************************/
/*                                                                   */
/* Function Name:  main                                              */
/*                                                                   */
/* Description:     Main routine                                     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/* Main routine                                                      */
/* Check parameter.                                                  */
/* Issue MQGET on input queue and call correct message procedure.    */
/*                                                                   */
/*                                                                   */
/* Input Parameters:                                                 */
/*       none                                                        */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:   0                                                      */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* Exit Normal:       0                                              */
/*                                                                   */
/* Exit Error:        0                                              */
/*                                                                   */
/*********************************************************************/

int main(int argc, char **argv)
{
	                          /* Start of program               */

 GetParm(argc, argv);             /* analyse parameters             */
	
 buflen = sizeof(buffer);         /* buffer length for MQGET        */

 /*******************************************************************/
 /*                                                                 */
 /*   This sample includes an explicit connect (MQCONN),            */
 /*   which isn't required on all MQSeries platforms,               */
 /*   because it's intended to be portable                          */
 /*                                                                 */
 /*******************************************************************/
 MQCONN(QMName,                   /* queue manager                  */
	 &Hcon,                   /* connection handle              */
	 &CompCode,               /* completion code                */
	 &CReason);               /* reason code                    */
	
 InquireQmgr(QMName);           /* Get Queue manager name           */

	                        /* write header and queue name etc. */
 printf("Sample AMQSLNK0 start\nQueue name=%.48s.\n"
	"Queue manager name=%.48s.\n",iQueue,QMName);

 /* report reason and stop if it failed                             */
 if (CompCode == MQCC_FAILED)
  {
    printf("MQCONN ended with reason code %ld\n", CReason);
    exit(CReason);
  }

 /*******************************************************************/
 /*                                                                 */
 /*   Open the input queue for shared input                         */
 /*                                                                 */
 /*******************************************************************/
 memcpy(odI.ObjectName,           /* name of input queue            */
	 iQueue, MQ_Q_NAME_LENGTH);
 O_options = MQOO_INPUT_SHARED    /* open queue for shared input    */
	    + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping    */
 MQOPEN(Hcon,                     /* connection handle              */
	 &odI,                    /* object descriptor for queue    */
	 O_options,               /* open options                   */
	 &Hinp,                   /* object handle                  */
	 &CompCode,               /* MQOPEN completion code         */
	 &Reason);                /* reason code                    */

 /* report reason if any; stop if it failed                         */
 if (Reason != MQRC_NONE)
  {
    printf("MQOPEN (input) ended with reason code %ld\n", Reason);
  }

 if (CompCode == MQCC_FAILED)
  {
    exit(Reason);
  };


 while (CompCode != MQCC_FAILED)
  {

   printf("Waiting for message ! \n");
	
	                          /* set up MQGET options           */
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
   gmo.Options |= MQGMO_WAIT+MQGMO_CONVERT;
   gmo.WaitInterval = MQWI_UNLIMITED;


   /*****************************************************************/
   /*                                                               */
   /*   In order to read the messages in sequence, MsgId and        */
   /*   CorrelID must have the default value.  MQGET sets them      */
   /*   to the values in the message it returns, so re-initialise   */
   /*   them before every call                                      */
   /*                                                               */
   /*****************************************************************/
   memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
   memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

   MQGET(Hcon,                    /* connection handle              */
	 Hinp,                    /* object handle                  */
	 &md,                     /* message descriptor             */
	 &gmo,                    /* GET options                    */
	 buflen,                  /* buffer length                  */
	 &buffer,                 /* message buffer                 */
	 &messlen,                /* message length                 */
	 &CompCode,               /* completion code                */
	 &Reason);                /* reason code                    */

   printf("MQGET reason code=%ld, messlen=%d, function=%.3s.\n",
	  Reason, messlen, buffer.function);


   /* report reason if any                                          */
   if (Reason != MQRC_NONE)
    {
     printf("MQGET ended with reason code %ld\n", Reason);
     return(1);                   /* MQGET failed                   */
    };


   /* Is it a put message request ?                                 */
   if (!strncmp(buffer.function,"Msg",3))
    {
     if (!Msg_call())             /* analyse "Msg" request          */
      continue;
     else break;
    };                            /* end of Msg request             */
	

   continue;                      /* Invalid message                */
  };                              /* === end of while ===           */
	
 C_options = 0;                   /*  no close options              */
 MQCLOSE(Hcon,                    /*  connection handle             */
	 &Hinp,                   /*  object handle                 */
	 C_options,               /*                                */
	 &CompCode,               /* completion code                */
	 &Reason);                /* reason code                    */
	

 /*******************************************************************/
 /*                                                                 */
 /*   Disconnect from MQM  (unless previously connected)            */
 /*                                                                 */
 /*******************************************************************/
 if (CReason != MQRC_ALREADY_CONNECTED)
  {
   MQDISC(&Hcon,                  /* connection handle              */
	  &CompCode,              /* completion code                */
	  &Reason);               /* reason code                    */

   /* report reason, if any                                         */
   if (Reason != MQRC_NONE)
    {
      printf("MQDISC ended with reason code %ld\n", Reason);
    }
  };

 /*******************************************************************/
 /*                                                                 */
 /* END OF AMQSLNK0                                                 */
 /*                                                                 */
 /*******************************************************************/
 printf("Sample AMQSLNK0 end\n");
 return(0);
}                                 /* end of main                    */
