/* @(#) include/amqsvmha.h, samples, nt200, nt200-csd08      1.6.1.2 96/02/14 11:22:50 */
 /********************************************************************/
 /*                                                                  */
 /* Module Name: AMQSVMHA.H                                          */
 /*                                                                  */
 /* Description: Sample Data Conversion macros for use in            */
 /*              conjunction with the Data Conversion                */
 /*              exit-generation utility program.                    */
 /*                                                                  */
 /*                                                                  */
 /* Licensed Materials - Property of IBM                             */
 /* 63H9336                                                          */
 /* (C) Copyright IBM Corp. 1994,1996.                               */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*   AMQSVMHA is a sample collection of macros to resolve the       */
 /*   macro calls generated by the Data Conversion exit-generation   */
 /*   utility.                                                       */
 /*     These macros make references to variables declared in the    */
 /*   sample Data Conversion exit routine AMQSVFCX.C.                */
 /*     The macros provided are:                                     */
 /*                                                                  */
 /*     AlignShort()                                                 */
 /*     AlignLong()                                                  */
 /*     ConvertByte()                                                */
 /*     ConvertChar()                                                */
 /*     ConvertVarChar()                                             */
 /*     ConvertShort()                                               */
 /*     ConvertLong()                                                */
 /*                                                                  */
 /********************************************************************/

 /* Protect against accidental double inclusion */
 #ifndef AMQSVMHA_DEFINED
 #define AMQSVMHA_DEFINED

 /********************************************************************/
 /* Include files                                                    */
 /********************************************************************/

 #include <string.h>      /* For memcpy                              */
 #include <cmqc.h>        /* Message Queueing Interface definitions  */
 #include <cmqxc.h>       /* MQI exit-related definitions            */

 /********************************************************************/
 /* Macros                                                           */
 /********************************************************************/

 /********************************************************************/
 /* CheckSpace(n, size, rcode)                                       */
 /*     n is the number of elements in the field                     */
 /*     size is the number of bytes in one element                   */
 /*     rcode is the return code from this macro (only set on error) */
 /*   This macro is called by the following macros to check that     */
 /*   there is enough space remaining in both the input and output   */
 /*   buffers to convert the next field.  If there is not, then it   */
 /*   reduces the value of n accordingly and sets rc to the          */
 /*   appropriate return code.                                       */
 /*   In some cases it is not an error if the input buffer is too    */
 /*   small to contain an entire message of the current format. For  */
 /*   example an application browsing the first 100 bytes of each    */
 /*   message on a queue where some of the messages are of a format  */
 /*   of more than 100 bytes. The CheckSpace macro uses the          */
 /*   MQxx_EXPECT_TRUNCATED_INPUT to identify if the data conversion */
 /*   routine is sympathetic to a truncated input buffer. If so then */
 /*   we expect the conversion routine to convert up to the end of   */
 /*   the last complete field. It's the applications business        */
 /*   whether it can then cope with the truncated message.           */
 /*   The value of rc depends on whether or not the original message */
 /*   was truncated and on whether or not the requesting application */
 /*   will accept a truncated message.                               */
 /********************************************************************/
 #ifdef MQxx_EXPECT_TRUNCATED_INPUT
 #define CheckSpace(n, size, rcode)                \
 if ((in_lastbyte - *in_cursor + 1) < n*size)      \
 {                                                 \
   n = (in_lastbyte - *in_cursor + 1) / size;      \
 }                                                 \
 if ((out_lastbyte - *out_cursor + 1) < n*size)    \
 {                                                 \
   n = (out_lastbyte - *out_cursor + 1) / size;    \
   rcode = (opts & MQGMO_ACCEPT_TRUNCATED_MSG)     \
           ? MQRC_TRUNCATED_MSG_ACCEPTED           \
           : MQRC_CONVERTED_MSG_TOO_BIG;           \
 }
 #else
 #define CheckSpace(n, size, rcode)                \
 if ((in_lastbyte - *in_cursor + 1) < n*size)      \
 {                                                 \
   n = (in_lastbyte - *in_cursor + 1) / size;      \
   rcode = (Reason == MQRC_TRUNCATED_MSG_ACCEPTED) \
           ? MQRC_TRUNCATED_MSG_ACCEPTED           \
           : MQRC_NOT_CONVERTED;                   \
 }                                                 \
 if ((out_lastbyte - *out_cursor + 1) < n*size)    \
 {                                                 \
   n = (out_lastbyte - *out_cursor + 1) / size;    \
   rcode = (opts & MQGMO_ACCEPT_TRUNCATED_MSG)     \
           ? MQRC_TRUNCATED_MSG_ACCEPTED           \
           : MQRC_CONVERTED_MSG_TOO_BIG;           \
 }
 #endif

 /********************************************************************/
 /* AlignShort()                                                     */
 /*   - Aligns the cursors suitably for a 2-byte field.              */
 /*     The default alignment rule is for all structures to be       */
 /*     packed.  If your structures follow a different alignment     */
 /*     rule then this macro will need changing appropriately.       */
 /********************************************************************/
 #define AlignShort()

 /********************************************************************/
 /* AlignLong()                                                      */
 /*   - Aligns the cursors suitably for a 4-byte field.              */
 /*     The default alignment rule is for all structures to be       */
 /*     packed.  If your structures follow a different alignment     */
 /*     rule then this macro will need changing appropriately.       */
 /********************************************************************/
 #define AlignLong()

 /********************************************************************/
 /* ConvertChar(cnumber)                                             */
 /*   - Converts the next cnumber bytes starting at in_cursor from   */
 /*     characters in MsgCCSID into characters in ReqCCSID starting  */
 /*     at out_cursor.                                               */
 /*       This macro should only be used for converting between SBCS */
 /*     character sets.   If DBCS is used and the field shrinks then */
 /*     no harm will result, but if expansion occurs then conversion */
 /*     will be halted with the Reason code MQRC_NOT_CONVERTED.      */
 /********************************************************************/
 #define ConvertChar(cnumber)           \
 {                                      \
   MQLONG cn, outlength, cc, rc, rc0;   \
   cn = cnumber; rc0 = MQRC_NONE;       \
   CheckSpace(cn, 1, rc0);              \
   MQXCNVC(hConn,                       \
           MQDCC_NONE,                  \
           MsgCCSID,                    \
           cn,                          \
  (PMQCHAR)*in_cursor,                  \
           ReqCCSID,                    \
           cn,                          \
  (PMQCHAR)*out_cursor,                 \
          &outlength,                   \
          &cc,                          \
          &rc);                         \
   if (rc0 == MQRC_NONE)                \
   {                                    \
     if (rc == MQRC_NONE)               \
     {                                  \
       *in_cursor  += cn;               \
       *out_cursor += cn;               \
     } else {                           \
       ReturnCode = MQRC_NOT_CONVERTED; \
       goto Fail;                       \
     }                                  \
   } else {                             \
     ReturnCode = rc0;                  \
     goto Fail;                         \
   }                                    \
 }

 /********************************************************************/
 /* ConvertVarChar                                                   */
 /*   - Converts characters to the end of the message.  This macro   */
 /*     can be used with DBCS fields.                                */
 /********************************************************************/
 #define ConvertVarChar()               \
 {                                      \
   MQLONG cn, ocn, cc, rc;              \
   cn = in_lastbyte - *in_cursor + 1;   \
   ocn = out_lastbyte - *out_cursor + 1;\
   MQXCNVC(hConn,                       \
           MQDCC_NONE,                  \
           MsgCCSID,                    \
           cn,                          \
  (PMQCHAR)*in_cursor,                  \
           ReqCCSID,                    \
           ocn,                         \
  (PMQCHAR)*out_cursor,                 \
          &ocn,                         \
          &cc,                          \
          &rc);                         \
   if (rc == MQRC_NONE)                 \
   {                                    \
     *in_cursor  += cn;                 \
     *out_cursor += ocn;                \
   } else if (rc == MQRC_TRUNCATED) {   \
     if (opts & MQGMO_ACCEPT_TRUNCATED_MSG) \
     {                                  \
       ReturnCode = MQRC_TRUNCATED_MSG_ACCEPTED; \
       *in_cursor  += cn;               \
       *out_cursor += ocn;              \
     } else {                           \
       ReturnCode = MQRC_NOT_CONVERTED; \
     }                                  \
   } else ReturnCode = rc;              \
   goto Fail;                           \
 }

 /********************************************************************/
 /* ConvertByte                                                      */
 /*   - Copies over bnumber bytes from input buffer to output buffer */
 /********************************************************************/
 #define ConvertByte(bnumber)           \
 {                                      \
   MQLONG bn = bnumber;                 \
   MQLONG rc0 = 0L;                     \
   CheckSpace(bn, 1, rc0);              \
   memcpy(*out_cursor, *in_cursor, bn); \
   *in_cursor  += bn;                   \
   *out_cursor += bn;                   \
   if (rc0 != MQRC_NONE)                \
   {                                    \
     ReturnCode = rc0;                  \
     goto Fail;                         \
   }                                    \
 }

 /********************************************************************/
 /* ConvertShort                                                     */
 /*   - Converts snumber 2-byte integers from MsgEncoding to         */
 /*     ReqEncoding                                                  */
 /********************************************************************/
 #define ConvertShort(snumber)               \
 { MQLONG EncIn, EncOut, i, sn;              \
   MQLONG rc0 = 0L;                          \
                                             \
   sn = snumber;                             \
   CheckSpace(sn, 2, rc0);                   \
   EncIn  = MsgEncoding & MQENC_INTEGER_MASK;\
   EncOut = ReqEncoding & MQENC_INTEGER_MASK;\
                                             \
   if (EncIn == EncOut)                      \
     { ConvertByte(2*sn); }                  \
   else                                      \
   if ((EncOut != MQENC_INTEGER_NORMAL) &&   \
       (EncOut != MQENC_INTEGER_REVERSED))   \
   { ReturnCode = MQRC_TARGET_INTEGER_ENC_ERROR;  \
     goto Fail; }                            \
   else                                      \
   if ((EncIn != MQENC_INTEGER_NORMAL) &&    \
       (EncIn != MQENC_INTEGER_REVERSED))    \
   { ReturnCode = MQRC_SOURCE_INTEGER_ENC_ERROR; \
     goto Fail; }                            \
   else for (i = 0; i < (sn); i++)           \
   { **out_cursor       = *(*in_cursor + 1); \
     *(*out_cursor + 1) = *(*in_cursor);     \
     *in_cursor  += 2;                       \
     *out_cursor += 2; }                     \
   if (rc0 != MQRC_NONE)                     \
   {                                         \
     ReturnCode = rc0;                       \
     goto Fail;                              \
   }                                         \
 }

 /********************************************************************/
 /* ConvertLong                                                      */
 /*   - Converts lnumber 4-byte integers from MsgEncoding to         */
 /*     ReqEncoding                                                  */
 /********************************************************************/
 #define ConvertLong(lnumber)                \
 { MQLONG EncIn, EncOut, i, ln;              \
   MQLONG rc0 = 0L;                          \
                                             \
   ln = lnumber;                             \
   CheckSpace(ln, 4, rc0);                   \
   EncIn  = MsgEncoding & MQENC_INTEGER_MASK;\
   EncOut = ReqEncoding & MQENC_INTEGER_MASK;\
                                             \
   if (EncIn == EncOut)                      \
     { ConvertByte(4*ln); }                  \
   else                                      \
   if ((EncOut != MQENC_INTEGER_NORMAL) &&   \
       (EncOut != MQENC_INTEGER_REVERSED))   \
   { ReturnCode = MQRC_TARGET_INTEGER_ENC_ERROR;  \
     goto Fail; }                            \
   else                                      \
   if ((EncIn != MQENC_INTEGER_NORMAL) &&    \
       (EncIn != MQENC_INTEGER_REVERSED))    \
   { ReturnCode = MQRC_SOURCE_INTEGER_ENC_ERROR; \
     goto Fail; }                            \
   else for (i = 0; i < (ln); i++)           \
   { **out_cursor       = *(*in_cursor + 3); \
     *(*out_cursor + 1) = *(*in_cursor + 2); \
     *(*out_cursor + 2) = *(*in_cursor + 1); \
     *(*out_cursor + 3) = **in_cursor;       \
     *in_cursor  += 4;                       \
     *out_cursor += 4; }                     \
   if (rc0 != MQRC_NONE)                     \
   {                                         \
     ReturnCode = rc0;                       \
     goto Fail;                              \
   }                                         \
 }

 #endif

